/*
 * Copyright (C) 1993-2002 William J. Poser.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>
#include "input.h"

/*
 * Read a line into buffer, with no special handling of comments
 * or quotes.	 
 * Returns the number of characters in the line, not counting the
 * terminating null. Returns:
 *	 
 *     ENDOFINPUT     if there is no more input
 *     BUFOVERFLOW    if the length of the line exceeds the buffer size. 
 *	 
 * These constants are defined in input.h.
 *
 */

#define DEFAULT 0
#define OVERFLOW 1

int
GetLine(FILE *fp,char *buf, int size)
{
  register int cnt = 0;
  register int c;
  int state = DEFAULT;
  static int GetLineDone = 0;
   
  if(GetLineDone){
    GetLineDone = 0;
    return(ENDOFINPUT);
  }
   
  while( (c = getc(fp)) != EOF){
    if(cnt == size){
      buf[cnt] = '\0';
      state = OVERFLOW;
    }
    if(c == '\n'){
      if(state == OVERFLOW) return(BUFOVERFLOW);
      else{
	buf[cnt] = '\0';
	return(cnt);
      }
    }
    else if(state == DEFAULT) buf[cnt++] = c;
  }
  GetLineDone = 1;
  if(state == OVERFLOW) return(BUFOVERFLOW);
  else{
    buf[cnt] = '\0';
    return(cnt);
  }
}


/*
 * Read a block of text terminated by an extra newline into a buffer.
 * Returns the number of characters in the block, ENDOFINPUT if there is no
 * more input, and BUFOVERFLOW if sufficient storage cannot be allocated.
 * The size of the input buffer, which may have been expanded, is assigned
 * to *size.
 */

char *
GetNNBlockRA(FILE *fp, char *buf, int *status, int *BufferSize, char t /* DUMMY */)
{
  register int cnt = 0;
  register int c;
  register int state = 0;
  static int done = 0;
   
  if(done){
    done = 0;
    *status =	ENDOFINPUT;
    return(buf);
  }
  while( (c = getc(fp)) != EOF){
    if(cnt == *BufferSize){
      *BufferSize = 2 * *BufferSize;
      buf = (char *) realloc( (void *) buf, (size_t) (*BufferSize * sizeof(char)));
      if(buf == (char *) 0){
	*status=BUFOVERFLOW;
	return(buf);
      }
    }
    switch(c){
    case '\n':
      if(state==0){
	buf[cnt++]='\n';
	state=1;
      }
      else if(state == 1){
	buf[cnt]='\0';
	*status=cnt;
	return(buf);
      }
      break;
    default:
      if(state==1) state=0;
      buf[cnt++]=c;
      break;
    }/* End of switch */
  } /* End of while */
	
  buf[cnt] = '\0';
  done = 1;
  *status=cnt;
  return(buf);
}


/*
 * Read a block of text terminated by a specified character
 * from a stream into a buffer, checking to make
 * sure that the size of the buffer is not exceeded.
 * Return the number of characters put into the buffer, not including
 * the terminating null, BUFOVERFLOW if storage cannot be allocated
 * or ENDOFINPUT if there is no more input.  	 
 */

char *
GetBlockSepCharRA(FILE *stream, char *buf, int *status, int *BufferSize, char t)
{
  register int c;
  register int itc;
  register int cnt = 0;
  static int done = 0;
   
  if(done){
    done = 0;
    *status=ENDOFINPUT;
    return(buf);
  }
   
  itc = (int) t;

  while( (c = getc(stream)) != EOF){
    if(cnt == *BufferSize){
      *BufferSize = 2 * *BufferSize;
      buf = (char *) realloc( (void *) buf, (size_t) (*BufferSize * sizeof(char)));
      if(buf == (char *) 0){
	*status=BUFOVERFLOW;
	return(buf);
      }
    }
    if(c == itc){
      buf[cnt]='\0';
      *status=cnt;
      return(buf);
    }
    else buf[cnt++] =c;
  }

  buf[cnt]='\0';
  done = 1;
  *status=cnt;
  return(buf);
}


/* 
 * Read a chunk of arbitrary length from a file, delimited by
 * a specified character.
 *
 * Return a pointer to the null-terminated string allocated, or null on failure
 * to allocate sufficient storage.
 * The length of the line is placed in the variable pointed to by
 * the third argument. The fourth argument indicates whether EOF was
 * encountered.
 */


#define INITLENGTH 64

char *
GetArbChunk(FILE *fp,char delim, long *LineLength,int *EndOfFile)
{
  char c;
  long Available;
  long CharsRead;
  char *Line;

  *EndOfFile=0;
  Available = INITLENGTH;
  CharsRead=0L;
  Line = (char *) malloc((size_t)Available);
  if(Line == (char *) 0) return (Line);

  while(1){
    c=getc(fp);
    if(c == delim){
      Line[CharsRead]='\0';
      *LineLength=CharsRead;
      return(Line);
    }
    if(c == EOF){
      Line[CharsRead]='\0';
      *EndOfFile=1;
      *LineLength=CharsRead;
      return(Line);
    }
    if(CharsRead == (Available-1)){ /* -1 because of null */
      Available += INITLENGTH/2;
      Line = (char *) realloc( (void *) Line, (size_t) (Available * sizeof (char)));
      if(Line == (char *) 0) return(Line);
    }
    Line[CharsRead++]=c;
  }
}

